@@ -10,7 +10,7 @@ from dateutil.tz import tzutc
 from dateutil.relativedelta import relativedelta
 from botocore.utils import parse_timestamp
 
-from awscli.compat import urlparse, RawConfigParser, StringIO
+from awscli.compat import is_windows, urlparse, RawConfigParser, StringIO
 from awscli.customizations import utils as cli_utils
 from awscli.customizations.commands import BasicCommand
 from awscli.customizations.utils import uni_print
@@ -98,16 +98,26 @@ class BaseLogin(object):
         raise NotImplementedError('get_commands()')
 
 
-class NuGetLogin(BaseLogin):
+class NuGetBaseLogin(BaseLogin):
     _NUGET_INDEX_URL_FMT = '{endpoint}v3/index.json'
 
+    # When adding new sources we can specify that we added the source to the
+    # user level NuGet.Config file. However, when updating an existing source
+    # we cannot be specific about which level NuGet.Config file was updated
+    # because it is possible that the existing source was not in the user
+    # level NuGet.Config. The source listing command returns all configured
+    # sources from all NuGet.Config levels. The update command updates the
+    # source in whichever NuGet.Config file the source was found.
+    _SOURCE_ADDED_MESSAGE = 'Added source %s to the user level NuGet.Config\n'
+    _SOURCE_UPDATED_MESSAGE = 'Updated source %s in the NuGet.Config\n'
+
     def login(self, dry_run=False):
         try:
             source_to_url_dict = self._get_source_to_url_dict()
         except OSError as ex:
             if ex.errno == errno.ENOENT:
                 raise ValueError(
-                    self._TOOL_NOT_FOUND_MESSAGE % 'nuget'
+                    self._TOOL_NOT_FOUND_MESSAGE % self._get_tool_name()
                 )
             raise ex
 
@@ -119,13 +129,13 @@ class NuGetLogin(BaseLogin):
         )
 
         if already_exists:
-            command = self._get_command(
+            command = self._get_configure_command(
                 'update', nuget_index_url, source_name
             )
-            verb = "Updated"
+            source_configured_message = self._SOURCE_UPDATED_MESSAGE
         else:
-            command = self._get_command('add', nuget_index_url, source_name)
-            verb = "Added"
+            command = self._get_configure_command('add', nuget_index_url, source_name)
+            source_configured_message = self._SOURCE_ADDED_MESSAGE
 
         if dry_run:
             dry_run_command = ' '.join([str(cd) for cd in command])
@@ -139,15 +149,14 @@ class NuGetLogin(BaseLogin):
                 stderr=self.subprocess_utils.PIPE
             )
         except subprocess.CalledProcessError as e:
-            uni_print('Failed to update the user level NuGet.Config\n')
+            uni_print('Failed to update the NuGet.Config\n')
             raise e
 
-        uni_print('%s source %s in the user level NuGet.Config\n'
-                  % (verb, source_name))
+        uni_print(source_configured_message % source_name)
         self._write_success_message('nuget')
 
     def _get_source_to_url_dict(self):
-        # The response from 'nuget sources list' takes the following form:
+        # The response from listing sources takes the following form:
         #
         # Registered Sources:
         #   1.  Source Name 1 [Enabled]
@@ -171,7 +180,7 @@ class NuGetLogin(BaseLogin):
        #       https://source100.com/index.json
 
         response = self.subprocess_utils.check_output(
-            ['nuget', 'sources', 'list', '-format', 'detailed'],
+            self._get_list_command(),
             stderr=self.subprocess_utils.PIPE
         )
 
@@ -221,7 +230,25 @@ class NuGetLogin(BaseLogin):
         # NuGet.Config file, use the default source name.
         return default_name, False
 
-    def _get_command(self, operation, nuget_index_url, source_name):
+    def _get_tool_name(self):
+        raise NotImplementedError('_get_tool_name()')
+
+    def _get_list_command(self):
+        raise NotImplementedError('_get_list_command()')
+
+    def _get_configure_command(self, operation, nuget_index_url, source_name):
+        raise NotImplementedError('_get_configure_command()')
+
+
+class NuGetLogin(NuGetBaseLogin):
+
+    def _get_tool_name(self):
+        return 'nuget'
+
+    def _get_list_command(self):
+        return ['nuget', 'sources', 'list', '-format', 'detailed']
+
+    def _get_configure_command(self, operation, nuget_index_url, source_name):
         return [
             'nuget', 'sources', operation,
             '-name', source_name,
@@ -231,6 +258,36 @@ class NuGetLogin(BaseLogin):
         ]
 
 
+class DotNetLogin(NuGetBaseLogin):
+
+    def _get_tool_name(self):
+        return 'dotnet'
+
+    def _get_list_command(self):
+        return ['dotnet', 'nuget', 'list', 'source', '--format', 'detailed']
+
+    def _get_configure_command(self, operation, nuget_index_url, source_name):
+        command = ['dotnet', 'nuget', operation, 'source']
+
+        if operation == 'add':
+            command.append(nuget_index_url)
+            command += ['--name', source_name]
+        else:
+            command.append(source_name)
+            command += ['--source', nuget_index_url]
+
+        command += [
+            '--username', 'aws',
+            '--password', self.auth_token
+        ]
+
+        # Encryption is not supported on non-Windows platforms.
+        if not is_windows:
+            command.append('--store-password-in-clear-text')
+
+        return command
+
+
 class NpmLogin(BaseLogin):
 
     # On Windows we need to be explicit about the .cmd file to execute
@@ -402,7 +459,7 @@ password: {auth_token}'''
                 sys.stdout.write(os.linesep)
                 raise e
         else:
-            pypi_rc.readfp(StringIO(default_pypi_rc))
+            pypi_rc.read_string(default_pypi_rc)
 
         pypi_rc_stream = StringIO()
         pypi_rc.write(pypi_rc_stream)
@@ -450,6 +507,11 @@ class CodeArtifactLogin(BasicCommand):
             'login_cls': NuGetLogin,
             'namespace_support': False,
         },
+        'dotnet': {
+            'package_format': 'nuget',
+            'login_cls': DotNetLogin,
+            'namespace_support': False,
+        },
         'npm': {
             'package_format': 'npm',
             'login_cls': NpmLogin,
